#!/usr/bin/php -q
<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2004-2008 The Cacti Group                                 |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | Cacti: The Complete RRDTool-based Graphing Solution                     |
 +-------------------------------------------------------------------------+
 | This code is designed, written, and maintained by the Cacti Group. See  |
 | about.php and/or the AUTHORS file for specific developer information.   |
 +-------------------------------------------------------------------------+
 | http://www.cacti.net/                                                   |
 +-------------------------------------------------------------------------+
*/

/* tick use required as of PHP 4.3.0 to accomodate signal handling */
declare(ticks = 1);

/*	display_help - displays the usage of the function */
function display_help () {
	print "Boost RRD Updater Process Control Version 2.0, Copyright 2005-2008 - Larry Adams\n\n";
	print "usage: poller_boost.php [-f | --force] [-d | --debug] [-h | -H | --help] [-v | -V | --version]\n\n";
	print "-f | --force     - Force the execution of a update process\n";
	print "-d | --debug     - Display verbose output during execution\n";
	print "-v -V --version  - Display this help message\n";
	print "-h -H --help     - display this help message\n";
}

function sig_handler($signo) {
	switch ($signo) {
		case SIGTERM:
		case SIGINT:
			cacti_log("WARNING: Boost Poller terminated by user", FALSE, "BOOST");

			/* tell the main poller that we are done */
			db_execute("REPLACE INTO settings (name, value) VALUES ('boost_poller_status', 'terminated - end time:" . date("Y-m-d G:i:s") ."')");

			exit;
			break;
		default:
			/* ignore all other signals */
	}

}

function output_rrd_data($start_time, $force = FALSE) {
	global $start, $max_run_duration, $config, $debug;

	include_once($config["base_path"] . "/lib/rrd.php");

	$boost_poller_status = read_config_option("boost_poller_status");
	$rrds_updates = 0;

	/* detect a process that has overrun it's warning time */
	if (substr_count($boost_poller_status, "running")) {
		$status_array = explode(":", $boost_poller_status);

		if (!empty($status_array[1])) {
			$previous_start_time = strtotime($status_array[1]);

			/* if the runtime was exceeded, allow the next process to run */
			if ($previous_start_time + $max_run_duration < $start_time) {
				cacti_log("WARNING: Detected Poller Boost Overrun, No Action Taken", FALSE, "BOOST SVR");
				$boost_poller_status = "overrun";
			}
		}
	}

	/* implement process lock control for boost */
	$lock_filename = read_config_option("boost_rrd_update_lockfile");
	if (!strlen($lock_filename)){
		if ($config["cacti_server_os"] != "win32") {
			$lock_filename = "/tmp/cacti-boost.lockf";
			db_execute("REPLACE INTO settings (name, value) VALUES ('boost_rrd_update_lockfile', '$lock_filename')");
		}else{
			if (is_dir("c:/windows/temp")) {
				$lock_filename = "c:/windows/temp/cacti-boost.lockf";
			}else if (is_dir("c:/winnt/temp")) {
				$lock_filename = "c:/winnt/temp/cacti-boost.lockf";
			}else{
				$lock_filename = "c:/cacti-boost.lockf";
			}
			db_execute("REPLACE INTO settings (name, value) VALUES ('boost_rrd_update_lockfile', '$lock_filename')");
		}
	}
	$fplock = fopen($lock_filename, "w+");

	/* if we can not open the lock file, the server is already running */
	if (!$fplock || !flock($fplock, LOCK_EX|LOCK_NB, $eWouldBlock) || $eWouldBlock) {
		if ($debug){
			cacti_log("DEBUG: Found lock, so another boost process is running");
		}
		return -1;
	}

	/* if the poller is not running, or has never run, start */
	if ((substr_count($boost_poller_status, "timeout")) ||
		(substr_count($boost_poller_status, "complete")) ||
		(substr_count($boost_poller_status, "terminated")) ||
		(substr_count($boost_poller_status, "abend")) ||
		($force) ||
		(empty($boost_poller_status))) {

		/* mark the boost server as running */
		db_execute("REPLACE INTO settings (name, value) VALUES ('boost_poller_status', 'running - start time:" . date("Y-m-d G:i:s") ."')");

		$current_time = date("Y-m-d G:i:s", $start_time);

		$rrdtool_pipe = rrd_init();

		while (1) {
			$rows = db_fetch_cell("SELECT count(*) FROM poller_output_boost WHERE time<='$current_time'");

			if ($rows > 0) {
				$rrds_updates += boost_process_poller_output($rrdtool_pipe, FALSE, "", $current_time);
			}else{
				break;
			}

			if ((time()-$start) > $max_run_duration) {
				cacti_log("FATAL: RRD On Demand Updater Exceeded Runtime Limits!!!");
				break;
			}
		}

		/* tell the main poller that we are done */
		db_execute("REPLACE INTO settings (name, value) VALUES ('boost_poller_status', 'complete - end time:" . date("Y-m-d G:i:s") ."')");

		rrd_close($rrdtool_pipe);
	}

	/* release the process lock */
	flock($fplock, LOCK_UN);
	fclose($fplock);
	unlink($lock_filename);

	return $rrds_updates;
}

function log_boost_statistics($rrd_updates) {
	global $start;

	/* take time and log performance data */
	list($micro,$seconds) = split(" ", microtime());
	$end = $seconds + $micro;

	$cacti_stats = sprintf(
		"Time:%01.4f " .
		"RRDUpdates:%s",
		round($end-$start,4),
		$rrd_updates);

	/* log to the database */
	 db_execute("REPLACE INTO settings (name,value) VALUES ('stats_boost', '" . $cacti_stats . "')");

	/* log to the logfile */
	cacti_log("BOOST STATS: " . $cacti_stats , TRUE, "SYSTEM");
}

/* do NOT run this script through a web browser */
if (!isset($_SERVER["argv"][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
	die("<br><strong>This script is only meant to run at the command line.</strong>");
}

/* We are not talking to the browser */
$no_http_headers = TRUE;

$dir = dirname(__FILE__);
chdir($dir);

if (strpos($dir, 'boost') !== FALSE) {
	chdir('../../');
}

/* include important functions */
if (file_exists("./include/global.php")) {
	include_once("./include/global.php");
} else {
	include_once("./include/config.php");
}
include_once($config["base_path"] . "/lib/poller.php");

/* get the boost polling cycle */
$max_run_duration = read_config_option("boost_rrd_update_max_runtime");

/* process calling arguments */
$parms = $_SERVER["argv"];
array_shift($parms);

$debug          = FALSE;
$forcerun       = FALSE;
$forcerun_maint = FALSE;

foreach($parms as $parameter) {
	@list($arg, $value) = @explode("=", $parameter);

	switch ($arg) {
	case "-d":
	case "--debug":
		$debug = TRUE;
		break;
	case "-f":
	case "--force":
		$forcerun = TRUE;
		break;
	case "-v":
	case "--version":
	case "-V":
	case "--version":
	case "--help":
	case "-h":
	case "-H":
		display_help();
		exit;
	default:
		print "ERROR: Invalid Parameter " . $parameter . "\n\n";
		display_help();
		exit;
	}
}

/* install signal handlers for UNIX only */
if (function_exists("pcntl_signal")) {
	pcntl_signal(SIGTERM, "sig_handler");
	pcntl_signal(SIGINT, "sig_handler");
}

/* take time and log performance data */
list($micro,$seconds) = split(" ", microtime());
$start = $seconds + $micro;

/* let's give this script lot's of memory, and let it run for ever */
ini_set("max_execution_time", "0");
ini_set("memory_limit", "2048M");

if ((read_config_option("boost_rrd_update_enable") == "on") || $forcerun) {
	/* turn on the system level updates as that is what dictates "on/off" */
	if ((!$forcerun) && (read_config_option("boost_rrd_update_system_enable") != "on")) {
		db_execute("REPLACE INTO settings (name,value)
			VALUES ('boost_rrd_update_system_enable','on')");
	}

	$seconds_offset = read_config_option("boost_rrd_update_interval") * 60;

	/* find out if it's time to collect device information */
	$last_run_time            = strtotime(read_config_option("boost_last_run_time"));
	$next_run_time            = strtotime(read_config_option("boost_next_run_time"));

	/* determine the next start time */
	$current_time = time();
	if (empty($last_run_time)) {
		/* since the poller has never run before, let's fake it out */
		$next_run_time = $current_time + $seconds_offset;
		db_execute("REPLACE INTO settings (name, value) VALUES ('boost_last_run_time', '" . date("Y-m-d G:i:s", $current_time) . "')");
	}else{
		$next_run_time = $last_run_time + $seconds_offset;
	}
	$time_till_next_run = $next_run_time - $current_time;

	/* determine if you must output boost table now */
	$max_records     = read_config_option("boost_rrd_update_max_records");
	$current_records = db_fetch_cell("SELECT count(*) FROM poller_output_boost");

	if (($time_till_next_run <= 0) || ($forcerun) || ($current_records >= $max_records) || ($next_run_time <= $current_time)) {
		db_execute("REPLACE INTO settings (name, value) VALUES ('boost_last_run_time', '" . date("Y-m-d G:i:s", $current_time) . "')");

		/* output all the rrd data to the rrd files */
		$rrds_updates = output_rrd_data($current_time, $forcerun);
		log_boost_statistics($rrds_updates);

		$next_run_time = $current_time + $seconds_offset;
	}

	/* store the next run time so that people understand */
	db_execute("REPLACE INTO settings (name, value) VALUES ('boost_next_run_time', '" . date("Y-m-d G:i:s", $next_run_time) . "')");
}else{
	/* turn on the system level updates as that is what dictates "off" */
	if (read_config_option("boost_rrd_update_system_enable") == "on") {
		db_execute("REPLACE INTO settings (name,value)
			VALUES ('boost_rrd_update_system_enable','off')");
	}

	$rows = db_fetch_cell("SELECT count(*) FROM poller_output_boost");

	if ($rows > 0) {
		/* determine the time to clear the table */
		$current_time = time();

		/* output all the rrd data to the rrd files */
		$rrds_updates = output_rrd_data($current_time, $forcerun);

		log_boost_statistics($rrds_updates);
	}
}

/* remove stale png's from the cache.  I consider png's stale afer 1 hour */
if ((read_config_option("boost_png_cache_enable") == "on") || $forcerun) {
	$cache_directory = read_config_option("boost_png_cache_directory");
	$remove_time = time() - 3600;

	$directory_contents = array();

	if ($handle = opendir($cache_directory)) {
		/* This is the correct way to loop over the directory. */
		while (FALSE !== ($file = readdir($handle))) {
			$directory_contents[] = $file;
		}

		closedir($handle);
	}

	/* remove age old files */
	if (sizeof($directory_contents)) {
		/* goto the cache directory */
		chdir($cache_directory);

		/* check and fry as applicable */
		foreach($directory_contents as $file) {
			if (is_writable($file)) {
				$modify_time = filemtime($file);
				if ($modify_time < $remove_time) {
					/* only remove jpeg's and png's */
					if ((substr_count(strtolower($file), ".png")) ||
						(substr_count(strtolower($file), ".jpg"))) {
						unlink($file);
					}
				}
			}
		}
	}
}

?>
